/*
	Word Twiddle
	JavaScript game engine
	Version 0.9
	Copyright 2011 by Andrew Krywaniuk
	
	License is hereby granted to freely distribute this game for personal and non-commercial use.
	For other uses, please contact the author.
*/


// In standalone mode, we load the whole dictionary and let the client choose the boards.
// In PC standalone mode, we are running on a PC, so we can't use any Ajax commands.
var standalone_mode = true;
var pc_standalone_mode = false;

// Server name for testing online play (remove this from the standalone edition).
//var g_server_root = "http://www.anagramfever.com/";
var g_server_root = "/";
//var g_server_root = "./";

// Info about the match we are currently playing.
var g_match_id = null;
var g_round_num = null;
var g_board_id = null;

// Number of rounds (currently only valid in standalone mode).
var g_num_rounds = 1;

// Match over (only valid in online mode).
var g_match_over = false;


// score_array - used to build a URI string.
var score_array = [
	"mi=",
	0, // 1 - match id
	"&rn=",
	0, // 3 - round num
	"&bi=",
	0, // 5 - board id
	"&rs=",
	0, // 7 - round score
	"&ms=",
	0, // 9 - max score
	"&fw=",
	0, // 11 - found words
	"&tw=",
	0, // 13 - total words
	"&tp=",
	0, // 15 - time played
	""
];


// submit_score - submit the results to the server.
function submit_score()
{
	score_array[1] = g_match_id;
	score_array[3] = g_round_num;
	score_array[5] = g_board_id;
	score_array[7] = g_running_score;
	score_array[9] = 0; // max score is not supported yet.
	score_array[11] = g_anagrams_found;
	score_array[13] = g_total_anagrams;
	score_array[15] = g_round_time - g_time_remaining;
	var post_data = score_array.join("");
	
dbg_msg("post_data = " + post_data);

	var url = g_server_root + "cgi-bin/submit_score.cgi?" + post_data;
	jx.load(url, on_submit_score_complete, "text", "POST");
}


// set_start_btn_game_complete - update the start button to make the user return back to
// the website.
function set_start_btn_game_complete(redir_url)
{
	var btn_start = $("btn_start");
	btn_start.title = "Return to the server";
	btn_start.value = "Done";
	function mk_cb(url) { return function() { window.location.href = url; } };
	btn_start.onclick = mk_cb(redir_url);
	
	btn_start.disabled = false;	
}


// update_start_button - update the start button to show either "start game"
// or "Next round".
function update_start_button()
{
	if (g_match_over) return false;

	var btn_start = $("btn_start");
	var show_next_round = false;
	
	if (standalone_mode)
	{
		if (g_round_num > 0 && g_round_num < g_num_rounds)
			show_next_round = true;
	}
	else
	{
		if (g_round_num > 1)
			show_next_round = true;
	}
	
	if (show_next_round)
	{
		btn_start.title = "Begin the next round";
		btn_start.value = "Next round";
	}
	else
	{
		btn_start.title = "Begin a new game";
		btn_start.value = "Start game";
	}
}


// set_start_btn_next_round - update the start button.
// The button is disabled until the round is loaded.
function set_start_btn_next_round()
{
	update_start_button();
	$("btn_start").disabled = true;
}


// on_submit_score_complete - callback when the score submission is complete.
function on_submit_score_complete(data)
{
	dbg_msg("submit score done - result=" + data);
	
	var a_lines = data.split("\n");
	var n = a_lines.length;
	
	// The first line is the error status.
	var result = parseInt(a_lines[0]);
	
	if (result)
	{
		var a_redir = a_lines[1].split("&");
		n = a_redir.length;
		
		if (n == 2 && a_redir[0] == "DONE")
		{
			// When the match is over, we just want to allow the user to view the result before returning
			// to the main page.
			g_match_over = true;
			set_start_btn_game_complete(a_redir[1]);
		}
		else if (n == 3)
		{
			g_match_id = parseInt(a_redir[0]);
			g_round_num = parseInt(a_redir[1]);
			g_board_id = parseInt(a_redir[2]);
			
			set_start_btn_next_round();

			load_round_from_server(g_board_id);
		}
		else
		{
			alert("bad response from server: " + a_redir);
		}
	}
	else
	{
		alert("Failed to submit score. Server returned: " + a_lines[1]);
	}
}

