/*
	Word Twiddle
	JavaScript game engine
	Version 0.9
	Copyright 2011 by Andrew Krywaniuk
	
	License is hereby granted to freely distribute this game for personal and non-commercial use.
	For other uses, please contact the author.
*/


// layout_cols - controls the number of columns in the word list.
var layout_cols = 6;

// unhighlighting operation.
var unhighlight_elem = null;
var unhighlight_timer = null;
var hide_res_timer = null;


var az_placeholders = [null, null, null, "___", "____", "_____", "______", "_______"];

function get_placeholder(wd)
{
	return az_placeholders[wd.length];
}


// Build an array to describe the striped indexes (assuming 6 columns)
function calc_striped_ordering(len)
{
	var a_ordering = [];
	var n = 0;
	
	var rows = Math.ceil(len/layout_cols);
//	var rem = rows * layout_cols - len;
	
	for (var i=0; i<rows; i++)
	{
		var a_inner = [];
		for (var j=0; j<layout_cols; j++)
		{
			var x = rows * j + i;
			if (x < len) a_inner.push(x);
		}
		
		a_ordering.push(a_inner);
	}
	
	return a_ordering;
}


// layout_words - layout the word entries on the screen.
// We make the word list a parameter so that we can show a default list when the page is first loaded.
function layout_words(olist)
{
	var warea = document.getElementById("word_list_area");
	
//	var olist = g_anagram_list;
	var len = olist.length;
	
	var a_ordering = calc_striped_ordering(len);
	
	var txt = '<table class="word_list"><tr>';
	
	for (var i=0; i<a_ordering.length; i++)
	{
		if (i > 0) txt += "</tr><tr>";
		
		var a_inner = a_ordering[i];
		
		for (var j=0; j<a_inner.length; j++)
		{
			var n = a_inner[j];
			txt += '<td class="word_list_entry"><span id="wd_' + n + '">' + get_placeholder(olist[n]) + '</span></td>';
		}
	}
	
	txt += "</tr></table>";
	warea.innerHTML = txt;
}


// highlight_word - briefly highlight the word that was just revealed.
function highlight_word(elem)
{
	// Clear any previous unhighlighting timer.
	if (unhighlight_timer)
	{
		window.clearTimeout(unhighlight_timer);
		unhighlight_word();
	}
	
	// Briefly highlight the word that was just revealed.
	elem.className = "just_found_word";
	unhighlight_elem = elem;
	unhighlight_timer = window.setTimeout(unhighlight_word, 500);
}


function unhighlight_word()
{
	unhighlight_elem.className = "found_word";
	unhighlight_timer = null;
}


// display_result_indicator - display a tick or X mark next to the input window.
function display_result_indicator(res)
{
	// It is too distracting to show an indicator for correct words. Let's show it
	// only for incorrect words.
	if (res == true) return;
	
	// Clear any previous timer.
	if (hide_res_timer)
	{
		window.clearTimeout(hide_res_timer);
		hide_result_indicator();
	}
	
	// Also display a check mark or X mark.
	elem = $("img_last_res");
	elem.src = res ? "res/tick_med.png" : "res/xmark_med.png";
	elem.style.visibility = "";
	hide_res_timer = window.setTimeout(hide_result_indicator, 300);
}

function hide_result_indicator()
{
	hide_res_timer = null;
	elem = $("img_last_res");
	elem.style.visibility = "hidden";
}


// reveal_word - when the user guesses a word, update its status and display it on the screen.
function reveal_word(idx)
{
	var elem_id = "wd_" + idx;
	var elem = document.getElementById(elem_id);
	elem.innerHTML = g_anagram_list[idx].toUpperCase();
	
	// Briefly highlight the word that was just revealed.
	highlight_word(elem);
}


// reveal_all_words - when the round is over, reveal all the words and highlight according to
// whether the user guessed them.
function reveal_all_words()
{
	for (var i=0; i<g_total_anagrams; i++)
	{
		var elem_id = "wd_" + i;
		var elem = $(elem_id);
		elem.innerHTML = g_anagram_list[i].toUpperCase();
		
		if (g_game_stats[i].found)
		{
			elem.className = "revealed_found_word";
		}
		else
		{
			elem.className = "revealed_unfound_word";
		}
	}
}


// update_game_controls - update the game controls to reflect the current game mode.
function update_game_controls()
{
	var dv_entry = $("text_entry_area");
	var dv_setup = $("game_setup_area");
	var btn_giveup = $("btn_giveup");
	
	if (g_game_in_progress)
	{
		dv_entry.style.display = "";
		dv_setup.style.display = "none";
		if (btn_giveup) btn_giveup.style.display = "";
		
		var eb = $("eb_input_word");
		eb.value = "";
	}
	else
	{
		update_game_desc();
		
		dv_entry.style.display = "none";
		dv_setup.style.display = "";
		if (btn_giveup) btn_giveup.style.display = "none";
	}
	
	
	// Some visual changes in online play vs. standalone mode.
	if (standalone_mode)
	{
		$("btn_settings").style.display = "";
	}
	else
	{
		$("btn_settings").style.display = "none";
	}
	
	update_start_button();
	update_round_display()
}


// update_round_display - update the round indicator in the top right-hand corner.
function update_round_display()
{
	if (standalone_mode && (g_num_rounds == 1))
	{
		$("sp_cur_round").innerHTML = "";
	}
	else if (standalone_mode)
	{
		$("sp_cur_round").innerHTML = "Round: " + g_round_num + "/" + g_num_rounds;
	}
	else
	{
		// In online mode, the GUI doesn't (as of Sept 2011) know how many rounds there are.
		$("sp_cur_round").innerHTML = "Round: " + g_round_num;
	}
}


// set_num_rounds - change the number of rounds in the game.
// (This implicitly sets the current round number back to 0.)
function set_num_rounds(num_rounds)
{
	g_num_rounds = num_rounds;
	g_round_num = 0;
	update_round_display();
}


// end_round - end the round
function end_round()
{
	if (g_game_timer)
	{
		clearTimeout(g_game_timer);
		g_game_timer = null;
	}

	if (unhighlight_timer)
	{
		clearTimeout(unhighlight_timer);
		unhighlight_timer = null;
	}

	g_game_in_progress = false;
	// We don't want to reset the timer display when the user finds all the words.	
//	g_time_remaining = 0;
	
	update_timer_display();
	update_game_controls();
	reveal_all_words();
	
	aggregate_round_stats();

	if (!standalone_mode)
	{	
		submit_score();
		
		// And wait for the callback to take us to the next round.
	}
}


// format_time - converts a seconds value to M:SS format.
function format_time(seconds_p)
{
	var mins = Math.floor(seconds_p/60);
	var secs = seconds_p % 60;
	var secs_str = (secs >= 10) ? ("" + secs) : ("0" + secs);
	
	return "" + mins + ":" + secs_str;
}

// update_timer_display - format the countdown timer display.
function update_timer_display()
{
	var sp_timer = document.getElementById("sp_timer");
	sp_timer.innerHTML = format_time(g_time_remaining);
}


// update_game_desc - update the configuration status bar
function update_game_desc()
{
	// In the original design, there is one span for the whole game description
	var sp_wd = document.getElementById("sp_game_desc");
	if (sp_wd)
	{
		sp_wd.innerHTML = "Word length: " + g_bingo_length + ", Round length: " + format_time(g_round_time);
		return;
	}
	
	// In the new design, there are separate spans for the two components.
	var sp_wl = $("sp_desc_bingo_len");
	var sp_rl = $("sp_desc_round_time");
	if (sp_wl && sp_rl)
	{
		sp_wl.innerHTML = "Word length: " + g_bingo_length;
		sp_rl.innerHTML = "Round length: " + format_time(g_round_time);
		return;
	}
}


// update_game_stats - update the runtime statistics (score, words found, etc)
function update_game_stats()
{
	var sp_wd = document.getElementById("sp_words_found");
	sp_wd.innerHTML = "" + g_anagrams_found + "/" + g_total_anagrams;

	var sp_score = document.getElementById("sp_score");
	sp_score.innerHTML = "" + g_running_score;
}


// update_scrambled_word - hide some letters in the scrambled word according to the current guess.
function update_scrambled_word(guess)
{
	var wd = g_scrambled_word;
	guess = guess.toUpperCase();
	
	var len = guess.length;
	for (var i=0; i<len; i++)
	{
		wd = wd.replace(guess[i], "&nbsp;");
	}
	
	var sp_swd = document.getElementById("sp_scrambled_word");
	sp_swd.innerHTML = wd;
}


// on_click_scramble_word - scramble the current anagram display.
function on_click_scramble_word()
{
	// Scrambling the word also clears the current guess.
	on_click_clear_word();
	
	var sp_swd = document.getElementById("sp_scrambled_word");
	g_scrambled_word = scramble_word(g_current_word);
	sp_swd.innerHTML = g_scrambled_word;
}


// on_click_last_word - reset the input box to the last valid word.
function on_click_last_word()
{
	$("eb_input_word").value = g_last_valid_word ? g_last_valid_word.toUpperCase() : "";
}

function on_click_clear_word()
{
	$("eb_input_word").value = "";
}


// on_enter_letter - handle input in the edit box.
function on_enter_letter(e)
{
	var charCode = (typeof e.which == "number") ? e.which : e.keyCode;
	var eb = document.getElementById("eb_input_word");
	
	// Process enter key.
	if (charCode == 13) // enter
	{
		var test_wd = eb.value.toLowerCase();
		eb.value = "";
		update_scrambled_word("");
		
		var res = process_guess(test_wd);
		display_result_indicator(res);
	}
	else if (charCode == 32) // space
	{
		on_click_scramble_word();
	}
	else if (charCode == 46) // .
	{
		on_click_last_word();
	}
	else if (charCode == 8) // backspace
	{
		var val = eb.value;
		var len = val.length;
		if (len)
		{
			eb.value = val.slice(0, len-1);
			update_scrambled_word(eb.value);
		}
	}
	else if ( (charCode >= 65 && charCode <= 90) // uppercase letter
		   || (charCode >= 97 && charCode <= 122) ) // lowercase letter
	{
		// Limit value length;
		if (eb.value.length < g_bingo_length)
		{
			// Only allow the user to input characters that exist in the scrambled word.
			var test_wd = eb.value + String.fromCharCode(charCode).toUpperCase();
			var test_wd_l = test_wd.toLowerCase();

			if (confirm_subword(test_wd_l, g_letter_map))
			{
				eb.value = test_wd;
				update_scrambled_word(test_wd);
			}
		}
	}
	
	return false;
}


// init_page_layout - update any layout parameters.
function init_page_layout()
{
	var num_cols = $("word_list_area").getAttribute("columns");
	if (num_cols)
	{
		layout_cols = parseInt(num_cols);
	}
	
}


// get_url_param - get one atom from the URL params.
// Code sample from http://www.netlobo.com/url_query_string_javascript.html
function get_url_param( name )
{
  name = name.replace(/[\[]/,"\\\[").replace(/[\]]/,"\\\]");
  var regexS = "[\\?&]"+name+"=([^&#]*)";
  var regex = new RegExp( regexS );
  var results = regex.exec( window.location.href );
  if( results == null )
    return null;
  else
    return results[1];
}


// wt_init_page - onload handler
function wt_init_page()
{
	// Detect if we are running on a PC.
	var prot = window.location.href.substr(0,5).toLowerCase();
	if (prot == "file:")
		pc_standalone_mode = true;

	// Extract any arguments from the URL.
	var match_id = get_url_param("mi");
	var round_num = get_url_param("rn");
	var board_id = get_url_param("bi");

	// In online play, the board id should always be specified.	
	if (board_id)
	{
		standalone_mode = false;
		
		g_match_id = match_id;
		g_round_num = round_num;
		g_board_id = board_id;
	}
	
	init_page_layout();
	init_session_stats();
	update_game_controls();
	setup_default_word_display();

	
	// In online play, the current word list *is* the dictionary. We don't need to care
	// about any words that are not part of this board.
	if (standalone_mode)
	{
		// Load the standalone game engine w/ the default dictionary.
		load_dictionary("ospd");
	}
	else
	{
		load_round_from_server(parseInt(board_id));
	}
}


window.onload = wt_init_page;

